@@ -19,14 +19,30 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 */
 
+
 #ifndef IN_UNIXINT
 #define NEED_MP_H
+#include <unistd.h>
+#include <string.h>
+#include <stdlib.h>
 #include "include.h"
 
+static void
+invoke_handler(int,int);
+
+
+#ifndef USIG2
 #include <signal.h>
 #include "usig.h"
-#include "arith.h"
+/*  #include "arith.h" */
 #endif
+#endif
+
+#ifdef USIG2
+#include USIG2
+#else
+
+
 
 /* these sstructure pointers would need their structures provided...
    so we just call them void */
@@ -37,12 +53,6 @@ EXTER
 unsigned long s4_neg_int[4],small_neg_int[3],small_pos_int[3];
 #endif
 
-tes()
-{void *buf[70];
- void **p=buf;
-#define XS(a) *p++ = (void *) (a)
-#include "usig2_aux.c"
-}
  
 /* 
    We have two mechanisms for protecting against interrupts.  1] We have a
@@ -78,14 +88,18 @@ struct save_for_interrupt{
 
 #undef SS1
 #undef RS1
-#define SS1(a,b) a = b ;
+#define SS1(a,b) a =  b ;
 #define RS1(a,b) b = a ;
 
            /* save objects in save_objects list  */   
 
 
  
-char signals_handled [] = {SIGINT,SIGUSR2,SIGUSR1,SIGIO,SIGALRM,0};
+char signals_handled [] = {SIGINT,SIGUSR2,SIGUSR1,SIGIO,SIGALRM,
+#ifdef OTHER_SIGNALS_HANDLED			   
+			   OTHER_SIGNALS_HANDLED
+#endif			   
+			   0};
 
 /* * in_signal_handler:   if not zero indicates we are running inside a signal
      handler, which may have been invoked at a random intruction, and so
@@ -107,7 +121,7 @@ char signals_handled [] = {SIGINT,SIGUSR2,SIGUSR1,SIGIO,SIGALRM,0};
       does the longjmp.
 
 
-/* If we invoke signal handling routines at a storage
+   If we invoke signal handling routines at a storage
    allocation pt, it is completely safe:  we should save
    some of the globals, but the freelists etc dont need
    to be saved.   pass: sig_safe to raise_pending.
@@ -130,34 +144,34 @@ char safety_required[]={XX,XX,XX,XX,XX,XX,XX,XX,
 			XX,XX,XX,XX,XX,XX,XX,XX,
 			XX,XX,XX,XX,XX,XX,XX,XX};
 
-int
-init_safety()
+void
+init_safety(void)
 { safety_required[SIGINT]=sig_try_to_delay;
   safety_required[SIGALRM]=sig_normal;
 }
   
 DO_INIT(init_safety();)
-DEFUN("SIGNAL-SAFETY-REQUIRED",int,sSsignal_safety_required,SI,2,2,
-      NONE,II,IO,OO,OO,
+DEFUN_NEW("SIGNAL-SAFETY-REQUIRED",object,sSsignal_safety_required,SI,2,2,
+	  NONE,OI,IO,OO,OO,(fixnum signo,fixnum safety),
       "Set the safety level required for handling SIGNO to SAFETY, or if \
 SAFETY is negative just return the current safety level for that \
 signal number.  Value of 1 means allow interrupt at any place not \
 specifically marked in the code as bad, and value of 2 means allow it \
 only in very SAFE places.")
 
-     
-     
-     (signo,safety)
 { if (signo > sizeof(safety_required))
     {FEerror("Illegal signo:~a.",1,make_fixnum(signo));}
   if (safety >=0) safety_required[signo] = safety;
-  return   safety_required[signo] ;
+  return small_fixnum(safety_required[signo]) ;
 }
      
 
 void
-main_signal_handler(signo, a,b)
-     int signo,a,b;
+#ifdef __MINGW32__
+main_signal_handler(int signo)
+#else    
+main_signal_handler(int signo, int a, int b)
+#endif    
 {  int allowed = signals_allowed;
 #ifdef NEED_TO_REINSTALL_SIGNALS
        signal(signo,main_signal_handler);
@@ -179,12 +193,12 @@ main_signal_handler(signo, a,b)
 
  }
 
-static void before_interrupt();
-static void after_interrupt();
+static void before_interrupt(struct save_for_interrupt *p, int allowed);
+static void after_interrupt(struct save_for_interrupt *p, int allowed);
 
 /* caller saves and restores the global signals_allowed; */
-invoke_handler(signo,allowed)
-     int signo,allowed;
+static void
+invoke_handler(int signo, int allowed)
 {struct save_for_interrupt buf;
  before_interrupt(&buf,allowed);
  signals_pending &= ~(signal_mask(signo));
@@ -199,9 +213,7 @@ invoke_handler(signo,allowed)
 
 int tok_leng;
 static void
-before_interrupt(p,allowed)
-   struct save_for_interrupt *p;
-     int allowed;
+before_interrupt(struct save_for_interrupt *p, int allowed)
 {int i;
  /* all this must be run in no interrupts mode */
  if ( allowed < sig_safe)
@@ -213,7 +225,7 @@ before_interrupt(p,allowed)
 	    { char *beg =  (char *) (p->free1[i]);
 	      object x = (object)beg;
 	      int amt = ad->tm_size;
-	      SS1(p->free2[i],F_LINK(p->free1[i]));
+	      SS1(p->free2[i],OBJ_LINK(p->free1[i]));
 	      ad->tm_nfree --;
 	      bcopy(beg ,&(p->buf[i]), amt);
 	      bzero(beg+8,amt-8);
@@ -223,11 +235,11 @@ before_interrupt(p,allowed)
 		  beg = (char *)x;
 		  x->d.m = 0;
 		  bzero(beg+8,amt-8);
-		  SS1(ad->tm_free,F_LINK(p->free2[i]));
+		  SS1(ad->tm_free,OBJ_LINK(p->free2[i]));
 		  ad->tm_nfree --;
 		}
 	      else
-		{ SS1(ad->tm_free,F_LINK(p->free1[i]));
+		{ SS1(ad->tm_free, OBJ_LINK(p->free1[i]));
 		}}
 	}}
    }
@@ -241,7 +253,8 @@ before_interrupt(p,allowed)
  SS1(p->ihs_topVAL,*ihs_top);
  { void **pp = p->save_objects;
 #undef XS
-#define XS(a) *pp++ = (void *) (a);
+ /* #define XS(a) *pp++ = (void *) (a); */
+#define XS(a) *pp++ =  * (void **) (&a); 
 #include "usig2_aux.c"
    if ((pp - (&(p->save_objects)[0])) >= (sizeof(p->save_objects)/sizeof(void *)))
      abort();
@@ -250,15 +263,13 @@ before_interrupt(p,allowed)
  p->token_st_dim = MINN(token->st.st_dim,tok_leng+1);
  if (p->token_st_dim < sizeof(p->token_buf))
    p->token_bufp = p->token_buf;
- else { p->token_bufp= OUR_ALLOCA(p->token_st_dim);}
+ else { p->token_bufp= (void *)OUR_ALLOCA(p->token_st_dim);}
  bcopy(token->st.st_self,p->token_bufp,p->token_st_dim);
   
 }
 
 static void
-after_interrupt(p,allowed)
-  struct save_for_interrupt *p;
-  int allowed;
+after_interrupt(struct save_for_interrupt *p, int allowed)
 {int i;
  /* all this must be run in no interrupts mode */
  if ( allowed < sig_safe)
@@ -271,14 +282,14 @@ after_interrupt(p,allowed)
 	    { char *beg =  (char *) (p->free1[i]);
 	      object x = (object)beg;
 	      int amt = ad->tm_size;
-	      RS1(p->free2[i],F_LINK(p->free1[i]));
+	      RS1(p->free2[i],(p->free1[i]));
 	      if (x->d.m) error("should not be free");
 	      bcopy(&(p->buf[i]),beg, amt);
 	      if (p->free2[i])
 		{ x = (object) p->free2[i];
 		  if (x->d.m) error("should not be free");
 		  x->d.m = FREE;
-		  F_LINK(F_LINK(ad->tm_free)) = current_fl;
+		  F_LINK(F_LINK(ad->tm_free)) = (long )current_fl;
 		  ad->tm_nfree += 2;
 		}
 	      else
@@ -298,8 +309,12 @@ after_interrupt(p,allowed)
   RS1(p->ihs_topVAL,*ihs_top);
  { void **pp = p->save_objects;
 #undef XS
-/* #define XS(a) ((void *)a) =  *pp++ */
-#define XS(a) a = (void *)(*pp++)
+
+ /*  #define XS(a) a = (void *)(*pp++)
+     We store back in the location 'a' the value we have saved. 
+  */
+ 
+#define XS(a) do { void **_p = (void **)(&a); *_p = (void *)(*pp++);}while(0)
 #include "usig2_aux.c"
  }
 
@@ -311,28 +326,27 @@ after_interrupt(p,allowed)
    and is suitable for inlining.
 */
 
-object
-MakeCons(a,b)
-     object a,b;
-{ struct typemanager*ad = &tm_table[t_cons];
-  object new = (object) ad->tm_free;
-  if (new == 0)
-    { new = alloc_object(t_cons);
-      new->c.c_car = a;
-      goto END;
-    }
+/* static object */
+/* MakeCons(object a, object b) */
+/* { struct typemanager*ad = &tm_table[t_cons]; */
+/*   object new = (object) ad->tm_free; */
+/*   if (new == 0) */
+/*     { new = alloc_object(t_cons); */
+/*       new->c.c_car = a; */
+/*       goto END; */
+/*     } */
       
-  new->c.c_car=a;
+/*   new->c.c_car=a; */
   /* interrupt here and before_interrupt will copy new->c into the
      C stack, so that a will be protected */
-  new->c.t=t_cons;
-  new->c.m= 0;
+/*   new->c.t=t_cons; */
+/*   new->c.m= 0; */
   /*  Make interrupt copy new out to the stack and then zero new.
       That way new is certainly gc valid, and its contents are protected.
       So the above three operations can occur in any order.
       */
 
-  { object tem  = F_LINK(new);
+/*   { object tem  = OBJ_LINK(new); */
     /*
       interrupt here and we see that before_interrupt must save the top of the
       free list AND the second thing on the Free list.  That way we will be ok
@@ -340,30 +354,28 @@ MakeCons(a,b)
       == 0, yet a gc happened in between.  An interrupt here when tem = 0 would
       mean the free list needs to be collected again by second gc.
       */
-    ad->tm_free = tem;
-  }
+/*     ad->tm_free = tem; */
+/*   } */
   /* Whew:  we got it safely off so interrupts can't hurt us now.  */
-  ad->tm_nfree --;
+/*   ad->tm_nfree --; */
   /* interrupt here and the cdr field will point to a f_link which is
      a 'free' and so gc valid.   b is still protected since
      it is in the stack or a regiseter, and a is protected since it is
      in new, and new is not free
      */
- END:
-  new->c.c_cdr=b;
-  return new;
-}
+/*  END: */
+/*   new->c.c_cdr=b; */
+/*   return new; */
+/* } */
 
 
 /* COND is the condition where this is raised.
    Might be sig_safe (eg at cons). */
    
 void
-raise_pending_signals(cond)
-     int cond;
+raise_pending_signals(int cond)
 {unsigned int allowed = signals_allowed ;
  if (cond == sig_use_signals_allowed_value)
-   { cond == allowed ;}
  if (cond == sig_none  || interrupt_enable ==0) return ;
  
  
@@ -373,7 +385,7 @@ raise_pending_signals(cond)
    if (pending)
      while(*p)
        { if (signal_mask(*p) & pending
-	     && cond >= safety_required[*p])
+	     && cond >= safety_required[(unsigned char)*p])
 	   {
 	     signals_pending &= ~(signal_mask(*p));
 	     if (*p == SIGALRM && cond >= sig_safe)
@@ -389,22 +401,22 @@ raise_pending_signals(cond)
  }}
 
 
-DEFUN("ALLOW-SIGNAL",int,fSallow_signal,SI,1,1,NONE,II,OO,OO,OO,
+DEFUN_NEW("ALLOW-SIGNAL",object,fSallow_signal,SI,1,1,NONE,OI,OO,OO,OO,(fixnum n),
       "Install the default signal handler on signal N")
-     (n)
-     int n;
-{int ma ;
- int ans = 0;
+
+{
+
  signals_allowed |= signal_mask(n);
  unblock_signals(n,n);
  /* sys v ?? just restore the signal ?? */
  if (our_signal_handler[n])
    {gcl_signal(n,our_signal_handler[n]);
-    return 1;
+    return make_fixnum(1);
   }
  else
-   return 0;
+   return make_fixnum(0);
 }
 
 
 
+#endif
